@@ -1,30 +1,30 @@
-/*
-  Copyright 2007-2008 MySQL AB
-
-  The MySQL Connector/ODBC is licensed under the terms of the
-  GPL, like most MySQL Connectors. There are special exceptions
-  to the terms and conditions of the GPL as it is applied to
-  this software, see the FLOSS License Exception available on
-  mysql.com.
-
-  This program is free software; you can redistribute it and/or modify
-  it under the terms of version 2 of the GNU General Public License as
-  published by the Free Software Foundation.
-
-  There are special exceptions to the terms and conditions of the GPL
-  as it is applied to this software. View the full text of the exception
-  in file LICENSE.exceptions in the top-level directory of this software
-  distribution.
-
-  This program is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
-
-  You should have received a copy of the GNU General Public License
-  along with this program; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-*/
+// Copyright (c) 2003, 2018, Oracle and/or its affiliates. All rights reserved.
+//
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License, version 2.0, as
+// published by the Free Software Foundation.
+//
+// This program is also distributed with certain software (including
+// but not limited to OpenSSL) that is licensed under separate terms,
+// as designated in a particular file or component or in included license
+// documentation. The authors of MySQL hereby grant you an
+// additional permission to link the program and your derivative works
+// with the separately licensed software that they have included with
+// MySQL.
+//
+// Without limiting anything contained in the foregoing, this file,
+// which is part of <MySQL Product>, is also subject to the
+// Universal FOSS Exception, version 1.0, a copy of which can be found at
+// http://oss.oracle.com/licenses/universal-foss-exception.
+//
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+// See the GNU General Public License, version 2.0, for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with this program; if not, write to the Free Software Foundation, Inc.,
+// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 
 #include "odbctap.h"
 
@@ -122,12 +122,12 @@ DECLARE_TEST(t_prep_buffer_length)
     rc = SQLExecute(hstmt);
     mystmt(hstmt,rc);
 
-    length= 10;    
+    length= 10;
 
     rc = SQLExecute(hstmt);
     mystmt(hstmt,rc);
 
-    length= 9;    
+    length= 9;
 
     rc = SQLExecute(hstmt);
     mystmt(hstmt,rc);
@@ -723,6 +723,7 @@ DECLARE_TEST(t_prepare)
 
     rc = SQLNumResultCols(hstmt,&pccol);
     mystmt(hstmt,rc);
+    is_num(pccol, 3);
 
     rc = SQLBindCol(hstmt,1,SQL_C_LONG,&nodata,0,&nlen);
     mystmt(hstmt,rc);
@@ -745,7 +746,7 @@ DECLARE_TEST(t_prepare)
     rc = SQLFetch(hstmt);
     mystmt(hstmt,rc);
 
-    fprintf(stdout," outdata: %d, %s(%ld)\n", nodata,szodata,nlen);
+    fprintf(stdout," outdata: %d, %s(%ld)\n", nodata,szodata,(long)nlen);
     my_assert(nodata == 200);
 
     rc = SQLFetch(hstmt);
@@ -1077,6 +1078,211 @@ DECLARE_TEST(t_bug29871)
 }
 
 
+/**
+  Bug #67340: Memory leak in 5.2.2(w) ODBC driver
+              causes Prepared_stmt_count to grow
+*/
+DECLARE_TEST(t_bug67340)
+{
+  SQLCHAR *param= (SQLCHAR *)"1";
+  SQLCHAR     data[255]= "abcdefg";
+  SQLLEN paramlen= 7;
+  int i, stmt_count= 0;
+
+  ok_sql(hstmt, "DROP TABLE IF EXISTS t_bug67340");
+  ok_sql(hstmt, "CREATE TABLE t_bug67340(id INT AUTO_INCREMENT PRIMARY KEY,"\
+                "vc VARCHAR(32))");
+
+  /* get the initial numnber of Prepared_stmt_count */
+  ok_sql(hstmt, "SHOW STATUS LIKE 'Prepared_stmt_count'");
+  ok_stmt(hstmt, SQLFetch(hstmt));
+  stmt_count= my_fetch_int(hstmt, 2);
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+
+  for(i=0; i < 100; i++)
+  {
+    ok_stmt(hstmt, SQLPrepare(hstmt, "INSERT INTO t_bug67340(id, vc) "\
+                                     "VALUES (NULL, ?)", SQL_NTS));
+    ok_stmt(hstmt, SQLBindParameter(hstmt, 1, SQL_PARAM_INPUT, SQL_C_CHAR,
+                                    SQL_CHAR, 0, 0, data, sizeof(data),
+                                    &paramlen));
+    ok_stmt(hstmt, SQLExecute(hstmt));
+
+    ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+    ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_UNBIND));
+    ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_RESET_PARAMS));
+  }
+
+  /* get the new numnber of Prepared_stmt_count */
+  ok_sql(hstmt, "SHOW STATUS LIKE 'Prepared_stmt_count'");
+  ok_stmt(hstmt, SQLFetch(hstmt));
+
+  /* check how much Prepared_stmt_count has increased */
+  is(!(my_fetch_int(hstmt, 2) - stmt_count > 1));
+
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+  ok_sql(hstmt, "DROP TABLE t_bug67340");
+  return OK;
+}
+
+
+/**
+  Bug #67702: Problem with BIT columns in MS Access
+*/
+DECLARE_TEST(t_bug67702)
+{
+  SQLCHAR data1[5]= "abcd";
+  char c1 = 1, c2= 0;
+  int id= 1;
+
+  SQLLEN paramlen= 0;
+
+  ok_stmt(hstmt, SQLExecDirect(hstmt, "drop table if exists bug67702",
+                                      SQL_NTS));
+
+  ok_stmt(hstmt, SQLExecDirect(hstmt, "create table bug67702"\
+                                      "(id int auto_increment primary key,"\
+                                      "vc varchar(32), yesno bit(1))",
+                                      SQL_NTS));
+
+  ok_stmt(hstmt, SQLExecDirect(hstmt, "INSERT INTO bug67702(id, vc, yesno)"\
+                                      "VALUES (1, 'abcd', 1)",
+                                      SQL_NTS));
+
+  /* Set parameter values here to make it clearer where each one goes */
+  c1= 0;
+  ok_stmt(hstmt, SQLBindParameter(hstmt, 1, SQL_PARAM_INPUT, SQL_C_BIT,
+                                  SQL_BIT, 1, 0, &c1, 0, NULL));
+
+  id= 1;
+  ok_stmt(hstmt, SQLBindParameter(hstmt, 2, SQL_PARAM_INPUT, SQL_C_LONG,
+                                  SQL_INTEGER, sizeof(id), 0, &id, 0, NULL));
+
+  paramlen= 4;
+  ok_stmt(hstmt, SQLBindParameter(hstmt, 3, SQL_PARAM_INPUT, SQL_C_CHAR,
+                                  SQL_VARCHAR, 4, 0, data1, 0, &paramlen));
+
+  c2= 1;
+  ok_stmt(hstmt, SQLBindParameter(hstmt, 4, SQL_PARAM_INPUT, SQL_C_BIT,
+                                  SQL_BIT, 1, 0, &c2, 0, NULL));
+
+  /* The prepared query looks exactly as MS Access does it */
+  ok_stmt(hstmt, SQLExecDirect(hstmt, "UPDATE `bug67702` SET `yesno`=?  "\
+                                      "WHERE `id` = ? AND `vc` = ? AND "\
+                                      "`yesno` = ?", SQL_NTS));
+  SQLFreeStmt(hstmt, SQL_CLOSE);
+
+  /* Now check the result of update the bit field should be set to 0 */
+  ok_stmt(hstmt, SQLExecDirect(hstmt, "SELECT `yesno` FROM `bug67702`", SQL_NTS));
+  ok_stmt(hstmt, SQLFetch(hstmt));
+  is(my_fetch_int(hstmt, 1) == 0);
+
+  SQLFreeStmt(hstmt, SQL_CLOSE);
+  ok_stmt(hstmt, SQLExecDirect(hstmt, "drop table if exists bug67702", SQL_NTS));
+  return OK;
+}
+
+
+/**
+  Bug #68243: Microsoft Access Crashes when Bit field updates
+*/
+DECLARE_TEST(t_bug68243)
+{
+  DECLARE_BASIC_HANDLES(henv1, hdbc1, hstmt1);
+  char c1 = 1, c2= 0;
+  int id= 1;
+
+  SQLLEN paramlen= 0;
+
+  ok_stmt(hstmt, SQLExecDirect(hstmt, "drop table if exists bug68243",
+                                      SQL_NTS));
+
+  ok_stmt(hstmt, SQLExecDirect(hstmt, "create table bug68243"\
+                                      "(id int primary key,"\
+                                      "yesno bit(1))",
+                                      SQL_NTS));
+
+  ok_stmt(hstmt, SQLExecDirect(hstmt, "INSERT INTO bug68243(id, yesno)"\
+                                      "VALUES (1, 1)",
+                                      SQL_NTS));
+
+  is(OK == alloc_basic_handles_with_opt(&henv1, &hdbc1, &hstmt1, NULL,
+                                        NULL, NULL, NULL, "NO_SSPS=1"));
+
+  /* Set parameter values here to make it clearer where each one goes */
+  c1= 0;
+  ok_stmt(hstmt1, SQLBindParameter(hstmt1, 1, SQL_PARAM_INPUT, SQL_C_BIT,
+                                  SQL_BIT, 1, 0, &c1, 0, NULL));
+
+  id= 1;
+  ok_stmt(hstmt1, SQLBindParameter(hstmt1, 2, SQL_PARAM_INPUT, SQL_C_LONG,
+                                  SQL_INTEGER, sizeof(id), 0, &id, 0, NULL));
+
+  c2= 1;
+  ok_stmt(hstmt1, SQLBindParameter(hstmt1, 3, SQL_PARAM_INPUT, SQL_C_BIT,
+                                  SQL_BIT, 1, 0, &c2, 0, NULL));
+
+  /* The prepared query looks exactly as MS Access does it */
+  ok_stmt(hstmt1, SQLExecDirect(hstmt1, "UPDATE `bug68243` SET `yesno`=?  "\
+                                      "WHERE `id` = ? AND `yesno` = ?",
+                                      SQL_NTS));
+  SQLFreeStmt(hstmt1, SQL_CLOSE);
+
+  /* Now check the result of update the bit field should be set to 0 */
+  ok_stmt(hstmt, SQLExecDirect(hstmt, "SELECT `yesno` FROM `bug68243`", SQL_NTS));
+  ok_stmt(hstmt, SQLFetch(hstmt));
+  is(my_fetch_int(hstmt, 1) == 0);
+
+  SQLFreeStmt(hstmt, SQL_CLOSE);
+  ok_stmt(hstmt, SQLExecDirect(hstmt, "drop table if exists bug68243", SQL_NTS));
+
+  free_basic_handles(&henv1, &hdbc1, &hstmt1);
+
+  return OK;
+}
+
+
+/**
+  Bug #67920: Non-compliant behavior of SQLMoreResults
+*/
+DECLARE_TEST(t_bug67920)
+{
+  ok_stmt(hstmt, SQLPrepare(hstmt, "SELECT 1", SQL_NTS));
+
+  expect_stmt(hstmt, SQLMoreResults(hstmt), SQL_NO_DATA);
+
+  SQLFreeStmt(hstmt, SQL_CLOSE);
+  return OK;
+}
+
+/*
+  Bug #31667091: ODBC prepared statements broken by changes
+  in behavior of @@sql_select_limit
+*/
+DECLARE_TEST(t_bug31667091)
+{
+  int id = 10;
+  ok_sql(hstmt,"DROP TABLE IF EXISTS bug31667091");
+  ok_sql(hstmt,"CREATE TABLE bug31667091 (id int)");
+  ok_sql(hstmt,"INSERT INTO bug31667091 VALUES (1), (2), (3)");
+  // Set row number limit to 1
+  ok_stmt(hstmt, SQLSetStmtAttr(hstmt, SQL_ATTR_MAX_ROWS, (SQLPOINTER)1, 0));
+  ok_sql(hstmt, "SELECT * FROM bug31667091");
+  // Make sure the limit is applied
+  is_num(my_print_non_format_result(hstmt), 1);
+
+  ok_stmt(hstmt, SQLPrepare(hstmt, "SELECT * FROM bug31667091 WHERE id < ?",
+                            SQL_NTS));
+  // Remove row number limit
+  ok_stmt(hstmt, SQLSetStmtAttr(hstmt, SQL_ATTR_MAX_ROWS, (SQLPOINTER)0, 0));
+  ok_stmt(hstmt, SQLBindParameter(hstmt, 1, SQL_PARAM_INPUT, SQL_C_LONG,
+                                  SQL_INTEGER, 0, 0, &id, 0, NULL));
+  ok_stmt(hstmt, SQLExecute(hstmt));
+  is_num(my_print_non_format_result(hstmt), 3);
+  ok_sql(hstmt,"DROP TABLE bug31667091");
+  return OK;
+}
+
 BEGIN_TESTS
   ADD_TEST(t_prep_basic)
   ADD_TEST(t_prep_buffer_length)
@@ -1092,6 +1298,11 @@ BEGIN_TESTS
   ADD_TEST(tmysql_bindparam)
   ADD_TEST(t_acc_update)
   ADD_TEST(t_bug29871)
+  ADD_TEST(t_bug67340)
+  ADD_TEST(t_bug67702)
+  ADD_TEST(t_bug68243)
+  ADD_TEST(t_bug67920)
+//  ADD_TODO(t_bug31667091)
 END_TESTS
 
 
