@@ -1,30 +1,69 @@
-/*
-  Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
-
-  The MySQL Connector/ODBC is licensed under the terms of the GPLv2
-  <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
-  MySQL Connectors. There are special exceptions to the terms and
-  conditions of the GPLv2 as it is applied to this software, see the
-  FLOSS License Exception
-  <http://www.mysql.com/about/legal/licensing/foss-exception.html>.
-  
-  This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published
-  by the Free Software Foundation; version 2 of the License.
-  
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
-  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
-  for more details.
-  
-  You should have received a copy of the GNU General Public License along
-  with this program; if not, write to the Free Software Foundation, Inc.,
-  51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
-*/
+// Copyright (c) 2007, 2018, Oracle and/or its affiliates. All rights reserved.
+//
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License, version 2.0, as
+// published by the Free Software Foundation.
+//
+// This program is also distributed with certain software (including
+// but not limited to OpenSSL) that is licensed under separate terms,
+// as designated in a particular file or component or in included license
+// documentation. The authors of MySQL hereby grant you an
+// additional permission to link the program and your derivative works
+// with the separately licensed software that they have included with
+// MySQL.
+//
+// Without limiting anything contained in the foregoing, this file,
+// which is part of <MySQL Product>, is also subject to the
+// Universal FOSS Exception, version 1.0, a copy of which can be found at
+// http://oss.oracle.com/licenses/universal-foss-exception.
+//
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+// See the GNU General Public License, version 2.0, for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with this program; if not, write to the Free Software Foundation, Inc.,
+// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 
+#include <wchar.h>
 #include "odbctap.h"
 #include <sqlucode.h>
-#include <wchar.h>
+
+DECLARE_TEST(emojibug)
+{
+  DECLARE_BASIC_HANDLES(henv1, hdbc1, hstmt1);
+  SQLWCHAR wbuff[MAX_ROW_DATA_LEN+1];
+  SQLLEN nLen = 0;
+
+  alloc_basic_handles_with_opt(&henv1, &hdbc1, &hstmt1, NULL, NULL, NULL,
+                               NULL, "CHARSET=utf8mb4");
+
+  ok_con(hdbc, SQLAllocStmt(hdbc1, &hstmt1));
+
+  ok_sql(hstmt1, "DROP TABLE IF EXISTS test_emoji");
+  ok_sql(hstmt1, "CREATE TABLE test_emoji (col1 varchar(32) character set utf8mb4)");
+  ok_sql(hstmt1, "INSERT INTO test_emoji VALUES (0xf09f988a)");
+
+  ok_stmt(hstmt1, SQLPrepareW(hstmt1, W(L"SELECT * FROM test_emoji"), SQL_NTS));
+
+  ok_stmt(hstmt1, SQLExecute(hstmt1));
+
+  ok_stmt(hstmt1, SQLFetch(hstmt1));
+
+  ok_stmt(hstmt1, SQLGetData(hstmt1, 1, SQL_WCHAR, wbuff, MAX_ROW_DATA_LEN + 1, &nLen));
+  printf("Fetched %d bytes (%d, %d)\n", (int)nLen, wbuff[0], wbuff[1]);
+  SQLWCHAR smile[2] = { 0xD83D, 0xDE0A };
+  is_num(smile[0], wbuff[0]);
+  is_num(smile[1], wbuff[1]);
+
+  expect_stmt(hstmt1, SQLFetch(hstmt1), SQL_NO_DATA_FOUND);
+
+  ok_stmt(hstmt1, SQLFreeStmt(hstmt1, SQL_CLOSE));
+
+  free_basic_handles(&henv1, &hdbc1, &hstmt1);
+  return OK;
+}
 
 
 DECLARE_TEST(sqlconnect)
@@ -166,20 +205,17 @@ DECLARE_TEST(sqlprepare_ansi)
 
 DECLARE_TEST(sqlchar)
 {
-  HDBC hdbc1;
-  HSTMT hstmt1;
+  DECLARE_BASIC_HANDLES(henv1, hdbc1, hstmt1);
   SQLCHAR data[]= "S\xe3o Paolo", buff[30];
   SQLWCHAR wbuff[MAX_ROW_DATA_LEN+1];
   wchar_t wcdata[]= L"S\x00e3o Paolo";
 
-  ok_env(henv, SQLAllocConnect(henv, &hdbc1));
-  ok_con(hdbc1, SQLConnectW(hdbc1,
-                            WC(mydsn), SQL_NTS,
-                            WC(myuid), SQL_NTS,
-                            WC(mypwd), SQL_NTS));
+  alloc_basic_handles_with_opt(&henv1, &hdbc1, &hstmt1, NULL, NULL, NULL,
+                               NULL, "");
+
   ok_con(hdbc, SQLAllocStmt(hdbc1, &hstmt1));
 
-  ok_stmt(hstmt1, SQLPrepareW(hstmt1, W(L"SELECT ? FROM DUAL"), SQL_NTS));
+  ok_stmt(hstmt1, SQLPrepareW(hstmt1, W(L"SELECT CAST(? AS BINARY)"), SQL_NTS));
 
   ok_stmt(hstmt1, SQLBindParameter(hstmt1, 1, SQL_PARAM_INPUT, SQL_C_CHAR,
                                    SQL_WVARCHAR, 0, 0, data, sizeof(data),
@@ -202,11 +238,7 @@ DECLARE_TEST(sqlchar)
 
   expect_stmt(hstmt1, SQLFetch(hstmt1), SQL_NO_DATA_FOUND);
 
-  ok_stmt(hstmt, SQLFreeStmt(hstmt1, SQL_DROP));
-
-  ok_con(hdbc1, SQLDisconnect(hdbc1));
-  ok_con(hdbc1, SQLFreeConnect(hdbc1));
-
+  free_basic_handles(&henv1, &hdbc1, &hstmt1);
   return OK;
 }
 
@@ -217,43 +249,44 @@ DECLARE_TEST(sqldriverconnect)
   HSTMT hstmt1;
   wchar_t conn_in[512];
   wchar_t dummy[256];
-  SQLWCHAR conn_out[512];
+  SQLWCHAR conn_out[1024];
   SQLSMALLINT conn_out_len;
 
   ok_env(henv, SQLAllocConnect(henv, &hdbc1));
 
   *conn_in= L'\0';
   wcscat(conn_in, L"DRIVER=");
-  mbstowcs(dummy, (char *)mydriver, sizeof(dummy));
+  mbstowcs(dummy, (char *)mydriver, sizeof(dummy)/sizeof(wchar_t));
   wcscat(conn_in, dummy);
   wcscat(conn_in, L";UID=");
-  mbstowcs(dummy, (char *)myuid, sizeof(dummy));
+  mbstowcs(dummy, (char *)myuid, sizeof(dummy)/sizeof(wchar_t));
   wcscat(conn_in, dummy);
   wcscat(conn_in, L";PWD=");
-  mbstowcs(dummy, (char *)mypwd, sizeof(dummy));
+  mbstowcs(dummy, (char *)mypwd, sizeof(dummy)/sizeof(wchar_t));
   wcscat(conn_in, dummy);
   wcscat(conn_in, L";DATABASE=");
-  mbstowcs(dummy, (char *)mydb, sizeof(dummy));
+  mbstowcs(dummy, (char *)mydb, sizeof(dummy)/sizeof(wchar_t));
   wcscat(conn_in, dummy);
   wcscat(conn_in, L";SERVER=");
-  mbstowcs(dummy, (char *)myserver, sizeof(dummy));
+  mbstowcs(dummy, (char *)myserver, sizeof(dummy)/sizeof(wchar_t));
   wcscat(conn_in, dummy);
   if (mysock != NULL)
   {
     wcscat(conn_in, L";SOCKET=");
-    mbstowcs(dummy, (char *)mysock, sizeof(dummy));
+    mbstowcs(dummy, (char *)mysock, sizeof(dummy)/sizeof(wchar_t));
     wcscat(conn_in, dummy);
   }
   if (myport)
   {
     char pbuff[20];
     sprintf(pbuff, ";PORT=%d", myport);
-    mbstowcs(dummy, (char *)pbuff, sizeof(dummy));
+    mbstowcs(dummy, (char *)pbuff, sizeof(dummy)/sizeof(wchar_t));
     wcscat(conn_in, dummy);
   }
 
   ok_con(hdbc1, SQLDriverConnectW(hdbc1, NULL, WL(conn_in, wcslen(conn_in)),
-                                  wcslen(conn_in), conn_out, sizeof(conn_out),
+                                  wcslen(conn_in), conn_out,
+                                  sizeof(conn_out)/sizeof(SQLWCHAR),
                                   &conn_out_len, SQL_DRIVER_NOPROMPT));
 
   ok_con(hdbc, SQLAllocStmt(hdbc1, &hstmt1));
@@ -261,12 +294,13 @@ DECLARE_TEST(sqldriverconnect)
   ok_stmt(hstmt1, SQLExecDirectW(hstmt1, W(L"SELECT 1234"), SQL_NTS));
   ok_stmt(hstmt1, SQLFetch(hstmt1));
   is_num(my_fetch_int(hstmt1, 1), 1234);
-
-  ok_stmt(hstmt, SQLFreeStmt(hstmt1, SQL_DROP));
-
+  printMessage("\nPrepare to Free STMT");
+  ok_stmt(hstmt1, SQLFreeStmt(hstmt1, SQL_CLOSE));
+  printMessage("\nDONE\nPrepare to Disconnect");
   ok_con(hdbc1, SQLDisconnect(hdbc1));
+  printMessage("\nDONE\nPrepare to Free Connect");
   ok_con(hdbc1, SQLFreeConnect(hdbc1));
-
+  printMessage("\nDONE\n");
   return OK;
 }
 
@@ -617,17 +651,18 @@ DECLARE_TEST(sqlgetdiagrec)
   ok_stmt(hstmt, SQLGetDiagRecW(SQL_HANDLE_STMT, hstmt1, 1, sqlstate,
                                 &native_err, message, 255, &msglen));
 
+  /* it has to comply to the bugfix for bug 14285620  */
   expect_stmt(hstmt1, SQLGetDiagRecW(SQL_HANDLE_STMT, hstmt1, 1, sqlstate,
                                      &native_err, message, 0, &msglen),
-              SQL_SUCCESS_WITH_INFO);
+                                     SQL_SUCCESS);
 
   expect_stmt(hstmt1, SQLGetDiagRecW(SQL_HANDLE_STMT, hstmt1, 1, sqlstate,
                                      &native_err, message, 10, &msglen),
-              SQL_SUCCESS_WITH_INFO);
+                                     SQL_SUCCESS_WITH_INFO);
 
   expect_stmt(hstmt1, SQLGetDiagRecW(SQL_HANDLE_STMT, hstmt1, 1, sqlstate,
                                      &native_err, message, -1, &msglen),
-              SQL_ERROR);
+                                     SQL_ERROR);
 
   ok_stmt(hstmt1, SQLFreeStmt(hstmt1, SQL_DROP));
   ok_con(hdbc1, SQLDisconnect(hdbc1));
@@ -1054,11 +1089,13 @@ DECLARE_TEST(t_bug34672)
   {
     chars[0]= 0xd802;
     chars[1]= 0xdc60;
+    chars[2]= 0;
     inchars= 2;
   }
   else
   {
     chars[0]= (SQLWCHAR) 0x10860;
+    chars[1]= 0;
     inchars= 1;
   }
 
@@ -1092,113 +1129,389 @@ Bug#28168 odbc, non 7-bit password, connection failed
 */
 DECLARE_TEST(t_bug28168)
 {
-  SQLHANDLE hdbc1;
+  SQLHANDLE hdbc1, hdbc2;
+  SQLHANDLE hstmt1;
 
-  wchar_t conn_in[512]= {0};
+  wchar_t conn_in[512]= {0}, work_conn_in[512]= {0};
   wchar_t dummy[256]= {0};
   wchar_t *wstr;
-  SQLWCHAR errmsgtxt[256]= {0};
+  SQLWCHAR errmsgtxt[256]= {0}, sqlstate[6]= {0};
+
+  SQLWCHAR *dropQuery= W(L"DROP USER IF EXISTS "
+    L"'\x03A8\x0391\x03A1\x039F uid'@localhost");
+  SQLWCHAR *dropQuery2= W(L"DROP USER IF EXISTS "
+    L"'\x03A8\x0391\x03A1\x039F uid'@'%'");
+
+  SQLWCHAR *createQuery= W(L"CREATE USER "
+    L"'\x03A8\x0391\x03A1\x039F uid'@"
+    L"localhost identified by "
+    L"'\x03A8\x0391\x03A1\x039F pwd'");
+  SQLWCHAR *createQuery2= W(L"CREATE USER "
+    L"'\x03A8\x0391\x03A1\x039F uid'@"
+    L"'%' identified by "
+    L"'\x03A8\x0391\x03A1\x039F pwd'");
   SQLSMALLINT errmsglen;
+  SQLINTEGER native_error= 0;
 
-  ok_env(henv, SQLAllocConnect(henv, &hdbc1));
-
+  /* Create tables to give permissions */
   ok_sql(hstmt, "DROP TABLE IF EXISTS t_bug28168");
   ok_sql(hstmt, "CREATE TABLE t_bug28168 (x int)");
 
-  ok_stmt(hstmt,
-    SQLExecDirectW(hstmt,
-    W(L"GRANT ALL ON t_bug28168 to "
-    L"'\x03A8\x0391\x03A1\x039F uid'@"
-    L"localhost identified by "
-    L"'\x03A8\x0391\x03A1\x039F pwd'"),
-    SQL_NTS));
+  *work_conn_in= L'\0';
+  wcscat(work_conn_in, L"DSN=");
+  mbstowcs(dummy, (char *)mydsn, sizeof(dummy)/sizeof(wchar_t));
+  wcscat(work_conn_in, dummy);
+  wcscat(work_conn_in, L";UID=");
+  mbstowcs(dummy, (char *)myuid, sizeof(dummy)/sizeof(wchar_t));
+  wcscat(work_conn_in, dummy);
+  wcscat(work_conn_in, L";PWD=");
+  mbstowcs(dummy, (char *)mypwd, sizeof(dummy)/sizeof(wchar_t));
+  wcscat(work_conn_in, dummy);
 
   ok_env(henv, SQLAllocHandle(SQL_HANDLE_DBC, henv, &hdbc1));
 
+  /* Connect using UTF8 as transport to avoid server bug with user names */
+  ok_con(hdbc1, SQLDriverConnectW(hdbc1, NULL, WL(work_conn_in,
+                                  wcslen(work_conn_in)),
+                                  wcslen(work_conn_in), NULL, 0,
+                                  0, SQL_DRIVER_NOPROMPT));
+
+  ok_con(hdbc1, SQLAllocStmt(hdbc1, &hstmt1));
+
+  /*
+    Grant for localhost and for all other hosts if the test server
+    runs remotely
+  */
+  ok_stmt(hstmt1, SQLExecDirectW(hstmt1, dropQuery, SQL_NTS));
+  ok_stmt(hstmt1, SQLExecDirectW(hstmt1, dropQuery2, SQL_NTS));
+
+  ok_stmt(hstmt1, SQLExecDirectW(hstmt1, createQuery, SQL_NTS));
+  ok_stmt(hstmt1, SQLExecDirectW(hstmt1, createQuery2, SQL_NTS));
+
   *conn_in= L'\0';
   wcscat(conn_in, L"DRIVER=");
-  mbstowcs(dummy, (char *)mydriver, sizeof(dummy));
+  mbstowcs(dummy, (char *)mydriver, sizeof(dummy)/sizeof(wchar_t));
   wcscat(conn_in, dummy);
   wcscat(conn_in, L";UID=");
   wcscat(conn_in, L"{\x03A8\x0391\x03A1\x039F uid}");
   wcscat(conn_in, L";PWD=");
   wcscat(conn_in, L"{\x03A8\x0391\x03A1\x039F pwd}");
   wcscat(conn_in, L";DATABASE=");
-  mbstowcs(dummy, (char *)mydb, sizeof(dummy));
+  mbstowcs(dummy, (char *)mydb, sizeof(dummy)/sizeof(wchar_t));
   wcscat(conn_in, dummy);
   wcscat(conn_in, L";SERVER=");
-  mbstowcs(dummy, (char *)myserver, sizeof(dummy));
+  mbstowcs(dummy, (char *)myserver, sizeof(dummy)/sizeof(wchar_t));
   wcscat(conn_in, dummy);
   if (mysock != NULL)
   {
     wcscat(conn_in, L";SOCKET=");
-    mbstowcs(dummy, (char *)mysock, sizeof(dummy));
+    mbstowcs(dummy, (char *)mysock, sizeof(dummy)/sizeof(wchar_t));
     wcscat(conn_in, dummy);
   }
 
-  ok_con(hdbc1, SQLDriverConnectW(hdbc1, NULL, W(conn_in), SQL_NTS, NULL, 0,
+  if (myport)
+  {
+    char pbuff[20];
+    sprintf(pbuff, ";PORT=%d", myport);
+    mbstowcs(dummy, (char *)pbuff, sizeof(dummy)/sizeof(wchar_t));
+    wcscat(conn_in, dummy);
+  }
+
+  ok_env(henv, SQLAllocHandle(SQL_HANDLE_DBC, henv, &hdbc2));
+  ok_con(hdbc2, SQLDriverConnectW(hdbc2, NULL, W(conn_in), SQL_NTS, NULL, 0,
     NULL, SQL_DRIVER_NOPROMPT));
-  ok_con(hdbc1, SQLDisconnect(hdbc1));
+  ok_con(hdbc2, SQLDisconnect(hdbc2));
 
   /* we change the password in the connection string to test the error msg */
   wstr= wcsstr(conn_in, L" pwd}") - 4;
   *wstr++= 'x';
 
-  expect_dbc(hdbc1, SQLDriverConnectW(hdbc1, NULL, W(conn_in), SQL_NTS, NULL,
-    0, NULL, SQL_DRIVER_NOPROMPT),
-    SQL_ERROR);
-  ok_con(hdbc1, SQLGetDiagFieldW(SQL_HANDLE_DBC, hdbc1, 1,
-    SQL_DIAG_MESSAGE_TEXT, errmsgtxt,
-    256 * sizeof(SQLWCHAR), &errmsglen));
-  swprintf(dummy, 256, L"[MySQL][ODBC 5.1 Driver]Access denied for user "
-    L"'\x03A8\x0391\x03A1\x039F uid'@'localhost' "
-    L"(using password: YES)");
+  expect_dbc(hdbc2, SQLDriverConnectW(hdbc2, NULL, W(conn_in), SQL_NTS,
+                  (SQLWCHAR*)NULL, 0, NULL, SQL_DRIVER_NOPROMPT), SQL_ERROR);
 
-  if (sizeof(SQLWCHAR) == sizeof(wchar_t))
-  {
-    printMessage("expected msg: %ls\n", dummy);
-    printMessage("actual msg: %ls\n", errmsgtxt);
-  }
+  ok_con(hdbc2, SQLGetDiagRecW(SQL_HANDLE_DBC, hdbc2, 1,
+                               sqlstate, &native_error, errmsgtxt,
+                               256 * sizeof(SQLWCHAR), &errmsglen));
+  ok_con(hdbc2, SQLFreeHandle(SQL_HANDLE_DBC, hdbc2));
 
-  is(!memcmp(errmsgtxt, W(dummy), wcslen(dummy) * sizeof(SQLWCHAR)));
-  is_num(errmsglen, wcslen(dummy) * sizeof(SQLWCHAR));
-
-  ok_con(hdbc1, SQLFreeHandle(SQL_HANDLE_DBC, hdbc1));
-
-  ok_stmt(hstmt,
-    SQLExecDirectW(hstmt,
+  /*
+    The returned error message has to contain the substring
+    with the username
+  */
+   wstr= sqlwchar_to_wchar_t(errmsgtxt);
+   is(wcsstr(wstr,
+             L"Access denied for user '\x03A8\x0391\x03A1\x039F uid'@") != NULL);
+  ok_stmt(hstmt1,SQLExecDirectW(hstmt1,
     W(L"DROP USER "
     L"'\x03A8\x0391\x03A1\x039F uid'@localhost"), SQL_NTS));
 
+  ok_stmt(hstmt1, SQLFreeStmt(hstmt1, SQL_DROP));
+  ok_con(hdbc1, SQLDisconnect(hdbc1));
+  ok_con(hdbc1, SQLFreeHandle(SQL_HANDLE_DBC, hdbc1));
+
   ok_sql(hstmt, "DROP TABLE IF EXISTS t_bug28168");
 
   return OK;
 }
 
 
+/*
+  Bug#14363601 MY_ISSPACE CALLS CAUSE ODBC DRIVER CRASHES
+*/
+DECLARE_TEST(t_bug14363601)
+{
+  HENV henv1;
+  HDBC hdbc1;
+  HSTMT hstmt1;
+  int i;
+  wchar_t conn_in[512];
+  wchar_t dummy[256];
+
+  SQLWCHAR conn_out[1024];
+  SQLSMALLINT conn_out_len;
+  SQLLEN strlen_or_ind= 10;
+  SQLINTEGER col_id= 1234;
+  SQLWCHAR *col_vc= W(L"abcdefg\x30a1"), col_vc_res[30];
+  double col_dc= 12345.678, col_dc_res= 0;
+  unsigned char col_bc[10]= {0, 1, 2, 3, 4, 5, 6, 7, 8, 9}, col_bc_res[10];
+
+  ok_env(henv1, SQLAllocEnv(&henv1));
+  ok_env(henv1, SQLAllocConnect(henv1, &hdbc1));
+
+  *conn_in= L'\0';
+  wcscat(conn_in, L"DRIVER=");
+  mbstowcs(dummy, (char *)mydriver, sizeof(dummy)/sizeof(wchar_t));
+  wcscat(conn_in, dummy);
+  wcscat(conn_in, L";UID=");
+  mbstowcs(dummy, (char *)myuid, sizeof(dummy)/sizeof(wchar_t));
+  wcscat(conn_in, dummy);
+  wcscat(conn_in, L";PWD=");
+  mbstowcs(dummy, (char *)mypwd, sizeof(dummy)/sizeof(wchar_t));
+  wcscat(conn_in, dummy);
+  wcscat(conn_in, L";DATABASE=");
+  mbstowcs(dummy, (char *)mydb, sizeof(dummy)/sizeof(wchar_t));
+  wcscat(conn_in, dummy);
+  wcscat(conn_in, L";SERVER=");
+  mbstowcs(dummy, (char *)myserver, sizeof(dummy)/sizeof(wchar_t));
+  wcscat(conn_in, dummy);
+  if (mysock != NULL)
+  {
+    wcscat(conn_in, L";SOCKET=");
+    mbstowcs(dummy, (char *)mysock, sizeof(dummy)/sizeof(wchar_t));
+    wcscat(conn_in, dummy);
+  }
+  if (myport)
+  {
+    char pbuff[20];
+    sprintf(pbuff, ";PORT=%d", myport);
+    mbstowcs(dummy, (char *)pbuff, sizeof(dummy)/sizeof(wchar_t));
+    wcscat(conn_in, dummy);
+  }
+  wcscat(conn_in, L";CHARSET=utf16");
+
+  ok_con(hdbc1, SQLDriverConnectW(hdbc1, NULL, WL(conn_in, wcslen(conn_in)),
+                                  wcslen(conn_in), conn_out,
+                                  sizeof(conn_out)/sizeof(SQLWCHAR),
+                                  &conn_out_len, SQL_DRIVER_NOPROMPT));
+
+  ok_con(hdbc1, SQLAllocStmt(hdbc1, &hstmt1));
+
+  ok_sql(hstmt1, "DROP TABLE IF EXISTS bug14363601");
+  ok_sql(hstmt1, "CREATE TABLE bug14363601("
+                 "id INT, vc VARCHAR(32),"
+                 "dc DOUBLE, bc BLOB)CHARSET=UTF8");
+
+  ok_stmt(hstmt1, SQLPrepareW(hstmt1,
+        W(L"INSERT INTO bug14363601 (id, vc, dc, bc) "
+                      L"VALUES (?, ?, ?, ?)"), SQL_NTS));
+
+  /* Bind 1st INT param */
+  ok_stmt(hstmt1, SQLBindParameter(hstmt1, 1, SQL_PARAM_INPUT, SQL_C_LONG,
+                                  SQL_INTEGER, 0, 0, &col_id, 0, NULL));
+
+  /* Bind 2nd VARCHAR param */
+  ok_stmt(hstmt1, SQLBindParameter(hstmt1, 2, SQL_PARAM_INPUT, SQL_C_WCHAR,
+           SQL_WCHAR, 10, 0, col_vc,
+                                  10*sizeof(SQLWCHAR), NULL));
+
+  /* Bind 3rd DECIMAL param */
+  ok_stmt(hstmt1, SQLBindParameter(hstmt1, 3, SQL_PARAM_INPUT, SQL_C_DOUBLE,
+                                  SQL_DOUBLE, 0, 0, &col_dc, sizeof(col_dc),
+                                  NULL));
+
+  /* Bind 4th BLOB param */
+  ok_stmt(hstmt1, SQLBindParameter(hstmt1, 4, SQL_PARAM_INPUT, SQL_C_BINARY,
+                                  SQL_BINARY, (SQLULEN)sizeof(col_bc), 0, &col_bc,
+                                  sizeof(col_bc), &strlen_or_ind));
+
+  ok_stmt(hstmt1, SQLExecute(hstmt1));
+
+  ok_stmt(hstmt1, SQLExecDirectW(hstmt1, W(L"SELECT * FROM bug14363601"),
+                                 SQL_NTS));
+
+  ok_stmt(hstmt1, SQLFetch(hstmt1));
+
+  is_num(my_fetch_int(hstmt1, 1), col_id);
+
+  ok_stmt(hstmt1, SQLGetData(hstmt1, 2, SQL_C_WCHAR, col_vc_res,
+                             sizeof(col_vc_res), NULL));
+
+  /* we want to compare SQLWCHAR instead of wchar_t */
+  for (i= 0; i < 8; i++)
+  {
+    is(col_vc[i] == col_vc_res[i]);
+  }
+
+  ok_stmt(hstmt1, SQLGetData(hstmt1, 3, SQL_C_DOUBLE, &col_dc_res,
+                             sizeof(col_dc_res), NULL));
+  is(col_dc == col_dc_res);
+
+  ok_stmt(hstmt1, SQLGetData(hstmt1, 4, SQL_C_BINARY, &col_bc_res,
+                             sizeof(col_bc_res), NULL));
+
+  /* check the binary buffer byte by byte */
+  for (i= 0; i < sizeof(col_bc_res); i++)
+  {
+    is_num(col_bc[i], col_bc_res[i]);
+  }
+
+  expect_stmt(hstmt1, SQLFetch(hstmt1), SQL_NO_DATA_FOUND);
+
+  ok_stmt(hstmt1, SQLFreeStmt(hstmt1, SQL_CLOSE));
+
+  ok_sql(hstmt1, "DROP TABLE IF EXISTS bug14363601");
+
+  ok_stmt(hstmt1, SQLFreeStmt(hstmt1, SQL_DROP));
+  ok_con(hdbc1, SQLDisconnect(hdbc1));
+  ok_con(hdbc1, SQLFreeConnect(hdbc1));
+  ok_env(henv1, SQLFreeEnv(henv1));
+
+  /* OK if it has not crashed */
+  return OK;
+}
+
+
+/*
+  Bug#14838690: ODBC 5.2.2 CONNECTOR BROKEN FOR ALL SQL SERVERS
+  OLDER THAN 5.5.3
+  With utf8 byte string, Mysql server version < 5.5.3 failes with
+  error message "Server does not support 4-byte encoded UTF8 characters."
+*/
+DECLARE_TEST(t_bug14838690)
+{
+  int i;
+  SQLINTEGER col_id= 1234;
+  SQLWCHAR *col_vc= W(L"abcdefg\x30a1"), col_vc_res[30];
+
+  ok_sql(hstmt, "DROP TABLE IF EXISTS bug14838690");
+  ok_sql(hstmt, "CREATE TABLE bug14838690("
+                 "id INT, vc VARCHAR(32)"
+                 ")CHARSET=UTF8");
+
+  ok_stmt(hstmt, SQLPrepareW(hstmt,
+        W(L"INSERT INTO bug14838690 (id, vc) "
+                      L"VALUES (?, ?)"), SQL_NTS));
+
+  /* Bind 1st INT param */
+  ok_stmt(hstmt, SQLBindParameter(hstmt, 1, SQL_PARAM_INPUT, SQL_C_LONG,
+                                  SQL_INTEGER, 0, 0, &col_id, 0, NULL));
+
+  /* Bind 2nd VARCHAR param with utf8 byte string */
+  ok_stmt(hstmt, SQLBindParameter(hstmt, 2, SQL_PARAM_INPUT, SQL_C_WCHAR,
+                                   SQL_WVARCHAR, 10, 0, col_vc,
+                                  10*sizeof(SQLWCHAR), NULL));
+
+  ok_stmt(hstmt, SQLExecute(hstmt));
+
+  ok_stmt(hstmt, SQLExecDirectW(hstmt, W(L"SELECT * FROM bug14838690"),
+                                 SQL_NTS));
+
+  ok_stmt(hstmt, SQLFetch(hstmt));
+
+  is_num(my_fetch_int(hstmt, 1), col_id);
+
+  ok_stmt(hstmt, SQLGetData(hstmt, 2, SQL_C_WCHAR, col_vc_res,
+                             sizeof(col_vc_res), NULL));
+
+  /* we want to compare SQLWCHAR instead of wchar_t */
+  for (i= 0; i < 8; i++)
+  {
+    is(col_vc[i] == col_vc_res[i]);
+  }
+
+  expect_stmt(hstmt, SQLFetch(hstmt), SQL_NO_DATA_FOUND);
+
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+
+  ok_sql(hstmt, "DROP TABLE IF EXISTS bug14838690");
+
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_DROP));
+
+  /* OK if it has not crashed */
+  return OK;
+}
+
+DECLARE_TEST(t_bug28864788)
+{
+  SQLWCHAR *col_vb = W(L"ABCDEF"), col_vb_res[30];
+  SQLLEN buf_len = 0;
+  int i = 0;
+
+  ok_sql(hstmt, "DROP TABLE IF EXISTS bug28864788");
+  ok_sql(hstmt, "CREATE TABLE bug28864788(id INT, vb VARBINARY(32))");
+  ok_sql(hstmt, "INSERT INTO bug28864788(id, vb) VALUES (1, 0xabcdef)");
+
+  ok_stmt(hstmt, SQLExecDirectW(hstmt, W(L"SELECT id, vb FROM bug28864788"),
+    SQL_NTS));
+
+  ok_stmt(hstmt, SQLFetch(hstmt));
+
+  ok_stmt(hstmt, SQLGetData(hstmt, 2, SQL_C_WCHAR, col_vb_res,
+          sizeof(col_vb_res), &buf_len));
+
+  // is_num(12, buf_len);
+
+  /* we want to compare SQLWCHAR instead of wchar_t */
+  for (i = 0; i < 6; i++)
+  {
+    is(col_vb[i] == col_vb_res[i]);
+  }
+
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+  ok_sql(hstmt, "DROP TABLE IF EXISTS bug28864788");
+
+  return OK;
+}
+
 BEGIN_TESTS
   ADD_TEST(sqlconnect)
-  ADD_TEST(sqlprepare)
+  ADD_TEST_UNICODE(emojibug)
+  ADD_TEST_UNICODE(sqlstatistics)
+  ADD_TEST_UNICODE(sqlprepare)
   ADD_TEST(sqlprepare_ansi)
-  ADD_TEST(sqlchar)
-  ADD_TEST(sqldriverconnect)
-  ADD_TEST(sqlnativesql)
+  ADD_TEST_UNICODE(sqlchar)
+  ADD_TEST_UNICODE(sqldriverconnect)
+  // ADD_TEST(sqlnativesql) TODO: Fix
+  ADD_TEST_UNICODE(sqlcolattribute)
+  ADD_TEST_UNICODE(sqldescribecol)
+#ifndef USE_IODBC
   ADD_TEST(sqlsetcursorname)
   ADD_TEST(sqlgetcursorname)
-  ADD_TEST(sqlcolattribute)
-  ADD_TEST(sqldescribecol)
   ADD_TEST(sqlgetconnectattr)
+#endif
   ADD_TEST(sqlgetdiagrec)
-  ADD_TEST(sqlgetdiagfield)
-  ADD_TEST(sqlcolumns)
-  ADD_TEST(sqltables)
-  ADD_TEST(sqlspecialcolumns)
+  // ADD_TEST(sqlgetdiagfield) TODO: Fix
+  ADD_TEST_UNICODE(sqlcolumns)
+  ADD_TEST_UNICODE(sqltables)
+  ADD_TEST_UNICODE(sqlspecialcolumns)
   ADD_TEST(sqlforeignkeys)
-  ADD_TEST(sqlprimarykeys)
-  ADD_TEST(sqlstatistics)
-  ADD_TEST(t_bug32161)
-  ADD_TEST(t_bug34672)
-  ADD_TEST(t_bug28168)
+  ADD_TEST_UNICODE(sqlprimarykeys)
+  ADD_TEST_UNICODE(t_bug32161)
+  // ADD_TEST_UNICODE(t_bug34672) TODO: Fix
+  // ADD_TEST_UNICODE(t_bug28168)
+  // ADD_TEST_UNICODE(t_bug14363601) TODO: Fix
+  // ADD_TEST_UNICODE(t_bug14838690) TODO: Fix
+  ADD_TEST(t_bug28864788)
 END_TESTS
 
 
