@@ -1,26 +1,30 @@
-/*
-  Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
-
-  The MySQL Connector/ODBC is licensed under the terms of the GPLv2
-  <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
-  MySQL Connectors. There are special exceptions to the terms and
-  conditions of the GPLv2 as it is applied to this software, see the
-  FLOSS License Exception
-  <http://www.mysql.com/about/legal/licensing/foss-exception.html>.
-  
-  This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published
-  by the Free Software Foundation; version 2 of the License.
-  
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
-  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
-  for more details.
-  
-  You should have received a copy of the GNU General Public License along
-  with this program; if not, write to the Free Software Foundation, Inc.,
-  51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
-*/
+// Copyright (c) 2007, 2018, Oracle and/or its affiliates. All rights reserved.
+//
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License, version 2.0, as
+// published by the Free Software Foundation.
+//
+// This program is also distributed with certain software (including
+// but not limited to OpenSSL) that is licensed under separate terms,
+// as designated in a particular file or component or in included license
+// documentation. The authors of MySQL hereby grant you an
+// additional permission to link the program and your derivative works
+// with the separately licensed software that they have included with
+// MySQL.
+//
+// Without limiting anything contained in the foregoing, this file,
+// which is part of <MySQL Product>, is also subject to the
+// Universal FOSS Exception, version 1.0, a copy of which can be found at
+// http://oss.oracle.com/licenses/universal-foss-exception.
+//
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+// See the GNU General Public License, version 2.0, for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with this program; if not, write to the Free Software Foundation, Inc.,
+// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 
 /**
   @file odbctap.h
@@ -28,6 +32,10 @@
   A basic interface for writing tests that produces TAP-compliant output.
 */
 
+/* We don't want ansi calls to be mapped to unicode counterparts, but that does not work */
+/* #define SQL_NOUNICODEMAP 1*/
+
+
 #ifdef HAVE_CONFIG_H
 # include <myconf.h>
 #endif
@@ -48,24 +56,35 @@
 #include <stdlib.h>
 #include <string.h>
 
+#ifdef _WIN32
+/* Driver does that - applications should do that too, to have same size of structures */
+# ifndef RC_INVOKED
+#  pragma pack(push, 1)
+# endif
+#endif
+
 #include <sql.h>
 #include <sqlext.h>
+#include <odbcinst.h>
+
+#ifdef _WIN32
+# ifndef RC_INVOKED
+#  pragma pack(pop)
+# endif
+#endif
 
 /* for clock() */
 #include <time.h>
 
-
 /* Get routines for helping with Unicode conversion. */
 #define ODBCTAP
 
-typedef unsigned int UTF32;
-typedef unsigned short UTF16;
-typedef unsigned char UTF8;
-
-#include "../util/unicode_transcode.c"
+#include "mysql_version.h"
+#include "../util/unicode_transcode.h"
 
 
-void printMessage(char *fmt, ...) {
+void printMessage(const char *fmt, ...)
+{
   va_list ap;
   va_start(ap, fmt);
   fprintf(stdout, "# ");
@@ -75,23 +94,73 @@ void printMessage(char *fmt, ...) {
 }
 
 
+/* Function that converts wchr_t string to char string for a debug output
+   Minimum guaranteed lenght of string it can convert is 2048/12=170. Stops if its
+   buffer cannot accomodate more characters from parameter string */
+const char * wstr4output(const wchar_t *wstr)
+{
+  size_t i;
+  static char str[2048];
+  char dbuf[16], *to= str;
+
+  for (i= 0; i < wcslen(wstr); ++i)
+  {
+    if (wstr[i] < 0x7f )
+    {
+      sprintf(dbuf, "%c", wstr[i]);
+    }
+    else
+    {
+      sprintf(dbuf, "[0x%x]", wstr[i]);
+    }
+
+    if (to - str + strlen(dbuf) + 1 < sizeof(str))
+    {
+      to+= sprintf(to, "%s", dbuf);
+    }
+    else
+    {
+      break;
+    }
+  }
+
+  return str;
+}
+
 #define MAX_NAME_LEN 255
 #define MAX_COLUMNS 500
 #define MAX_ROW_DATA_LEN 1000
 #define MYSQL_NAME_LEN 64
+#define MAX_MEM_BLOCK_ELEMENTS 100
 
-SQLCHAR *mydriver= (SQLCHAR *)"{MySQL ODBC 5.1 Driver}";
+#ifdef __WIN__
+#define _MY_NEWLINE "\r\n"
+#else
+#define _MY_NEWLINE "\n"
+#endif
+
+SQLCHAR *mydriver= (SQLCHAR *)"{MySQL ODBC 8.0 Driver}";
 SQLCHAR *mydsn= (SQLCHAR *)"test";
 SQLCHAR *myuid= (SQLCHAR *)"root";
 SQLCHAR *mypwd= (SQLCHAR *)"";
 SQLCHAR *mysock= NULL;
-int      myoption= 0, myport= 0;
+SQLCHAR *mydns_srv= NULL;
+int      myoption= 0, myport= 0, myenable_pooling= 0;
+SQLCHAR *my_str_options= (SQLCHAR *)""; /* String for additional connection options */
 SQLCHAR *myserver= (SQLCHAR *)"localhost";
 SQLCHAR *mydb= (SQLCHAR *)"test";
-SQLCHAR *test_db= (SQLCHAR *)"client_odbc_test";
+SQLCHAR *myauth= NULL;
+SQLCHAR *myplugindir= NULL;
 
-int g_nCursor;
-char g_szHeader[501];
+SQLCHAR *test_db= (SQLCHAR *)"client_odbc_test";
+/* Suffix is useful if a testsuite is run more than once */
+const SQLCHAR *testname_suffix= (SQLCHAR*)"";
+/* -1 means that the fact has to be established, 0 - ansi driver, 1 - unicode */
+int     unicode_driver= -1;
+#define REQUIRES_UNICODE_DRIVER if (unicode_driver == 0) skip("This testcase is designed for Unicode drivers only")
+#define REQUIRES_ANSI_DRIVER if (unicode_driver != 0) skip("This testcase is designed for ANSI drivers only")
+
+int      init_auth_plugin= 0;
 
 #ifndef OK
 # define OK 0
@@ -111,9 +180,11 @@ char g_szHeader[501];
 
 char *SKIP_REASON= NULL;
 
+#define USE_DRIVER (char *)-1
+
 typedef int (*test_func)(SQLHDBC, SQLHSTMT, SQLHENV);
 static void print_diag(SQLRETURN rc, SQLSMALLINT htype, SQLHANDLE handle,
-		       const char *text, const char *file, int line);
+           const char *text, const char *file, int line);
 
 /* Disable _attribute__ on non-gcc compilers. */
 #if !defined(__attribute__) && !defined(__GNUC__)
@@ -130,17 +201,29 @@ static void print_diag(SQLRETURN rc, SQLSMALLINT htype, SQLHANDLE handle,
              SQLHENV henv __attribute__((unused)))
 
 typedef struct {
-  char *name;
+  char      *name;
   test_func func;
-  int   expect;
+  int       expect;
+  int       required_driver_type;
 } my_test;
 
+
+typedef struct
+{
+  void *blk[MAX_MEM_BLOCK_ELEMENTS];
+  int  counter;
+} GCOLL;
+
+GCOLL gc_blk;
+
+
 #ifdef WIN32
 void test_timeout(int signum);
 HANDLE halarm= NULL;
 DWORD WINAPI win32_alarm(LPVOID arg)
 {
-  DWORD timeout= ((DWORD) arg) * 1000;
+  unsigned long long llarg = (unsigned long long)arg;
+  DWORD timeout= ((DWORD) llarg) * 1000;
   while (WaitForSingleObject(halarm, timeout) == WAIT_OBJECT_0);
   test_timeout(0);
   return 0;
@@ -148,12 +231,13 @@ DWORD WINAPI win32_alarm(LPVOID arg)
 #define ENABLE_ALARMS    int do_alarms= !getenv("DISABLE_TIMEOUT")
 #define RUN_TESTS_SIGNAL halarm= CreateEvent(NULL, FALSE, FALSE, NULL); \
                          if (do_alarms) \
-                           CreateThread(NULL, 0, win32_alarm, (LPVOID) 30, 0, NULL)
+                           CreateThread(NULL, 0, win32_alarm, (LPVOID) 1200, 0, NULL); \
+                         do_alarms= 0
 #define RUN_TESTS_ALARM (void) SetEvent(halarm)
 #else
 #define ENABLE_ALARMS    int do_alarms= !getenv("DISABLE_TIMEOUT")
 #define RUN_TESTS_SIGNAL (void)signal(SIGALRM, test_timeout)
-#define RUN_TESTS_ALARM  if (do_alarms) alarm(30)
+#define RUN_TESTS_ALARM  if (do_alarms) alarm(1200)
 #endif
 
 void mem_debug_init()
@@ -168,9 +252,31 @@ void mem_debug_init()
 #endif
 }
 
+/*
+  Initialize garbage collector
+  counter (or position)
+*/
+void mem_gc_init()
+{
+  gc_blk.counter= 0;
+}
+
+#define SQL_IS_SUCCESS(R) ((R == SQL_SUCCESS) || (R == SQL_SUCCESS_WITH_INFO))
+
+#define DECLARE_BASIC_HANDLES(E, C, S) SQLHENV E= NULL; \
+  SQLHDBC C= NULL; \
+  SQLHSTMT S= NULL
+
 #define BEGIN_TESTS my_test tests[]= {
-#define ADD_TEST(name) { #name, name, OK   },
-#define ADD_TODO(name) { #name, name, FAIL },
+#define ADD_TEST(name)          { #name, name, OK  , -1 },
+#define ADD_TEST_UNICODE(name)  { #name, name, OK  ,  1 },
+#define ADD_TEST_ANSI(name)     { #name, name, OK  ,  0 },
+#define ADD_TODO(name)          { #name, name, FAIL, -1 },
+#ifdef EXPOSE_TOFIX_TESTS
+# define ADD_TOFIX(name) { #name, name, OK,   -1 },
+#else
+# define ADD_TOFIX(name) { #name, name, FAIL, -1 },
+#endif
 #define END_TESTS }; \
 void test_timeout(int signum __attribute__((unused))) \
 { \
@@ -182,11 +288,12 @@ int main(int argc, char **argv) \
 { \
   SQLHENV  henv; \
   SQLHDBC  hdbc; \
-  SQLHSTMT hstmt; \
+  SQLHSTMT hstmt = NULL; \
   int      i, num_tests, failcnt= 0; \
   ENABLE_ALARMS; \
 \
   mem_debug_init(); \
+  mem_gc_init(); \
 \
   /* Set from environment, possibly overrided by command line */ \
   if (getenv("TEST_DSN")) \
@@ -199,8 +306,18 @@ int main(int argc, char **argv) \
     mypwd=  (SQLCHAR *)getenv("TEST_PASSWORD"); \
   if (getenv("TEST_SOCKET")) \
     mysock= (SQLCHAR *)getenv("TEST_SOCKET"); \
+  if (getenv("TEST_SERVER")) \
+    myserver= (SQLCHAR *)getenv("TEST_SERVER"); \
   if (getenv("TEST_PORT")) \
     myport= atoi(getenv("TEST_PORT")); \
+  if (getenv("TEST_ENABLE_POOLING")) \
+    myenable_pooling= atoi(getenv("TEST_ENABLE_POOLING")); \
+  if (getenv("TEST_DEFAULTAUTH")) \
+    myauth=  (SQLCHAR *)getenv("TEST_DEFAULTAUTH"); \
+  if (getenv("TEST_PLUGINDIR")) \
+    myplugindir=  (SQLCHAR *)getenv("TEST_PLUGINDIR"); \
+  if (getenv("TEST_DNS_SRV")) \
+    mydns_srv= (SQLCHAR *)getenv("TEST_DNS_SRV"); \
 \
   if (argc > 1) \
     mydsn= (SQLCHAR *)argv[1]; \
@@ -214,38 +331,46 @@ int main(int argc, char **argv) \
 #define SET_DSN_OPTION(x) \
   myoption= (x);
 
-#define RUN_TESTS \
+#define RUN_TESTS_ONCE \
   setbuf(stdout, NULL); \
   num_tests= sizeof(tests) / sizeof(tests[0]); \
   printf("1..%d\n", num_tests); \
 \
   RUN_TESTS_SIGNAL; \
 \
-  if (alloc_basic_handles(&henv, &hdbc, &hstmt) != OK) \
-    exit(1); \
 \
   for (i= 0; i < num_tests; i++ ) \
   { \
     int rc; \
-    RUN_TESTS_ALARM; \
-    rc= tests[i].func(hdbc, hstmt, henv); \
-    printf("%s %d - %s %s%s\n", \
+    if (tests[i].required_driver_type > -1 && tests[i].required_driver_type != unicode_driver) \
+    { \
+      printf("ok %d # SKIP This testcase is designed for %s drivers only\n", i+1, \
+             unicode_driver == 0 ? "Unicode" : "ANSI" ); \
+    } \
+    else \
+    { \
+      if (alloc_basic_handles(&henv, &hdbc, &hstmt) != OK) \
+        exit(1); \
+      RUN_TESTS_ALARM; \
+      rc= tests[i].func(hdbc, hstmt, henv); \
+      printf("%s %d - %s%s %s%s\n", \
            (rc == OK || rc == SKIP) ? "ok" : "not ok", \
            i + 1, \
-           tests[i].name, \
+           tests[i].name, testname_suffix, \
            (tests[i].expect == FAIL ? "# TODO" : \
             rc == SKIP ? "# SKIP " : ""), \
            SKIP_REASON ? SKIP_REASON : ""); \
-    if ((rc == FAIL) && (FAIL != tests[i].expect)) \
-      failcnt++; \
-    SKIP_REASON= NULL; /* Reset SKIP_REASON */ \
-    /* Re-allocate statement to reset all its properties. */ \
-    SQLFreeStmt(hstmt, SQL_DROP); \
-    SQLAllocHandle(SQL_HANDLE_STMT, hdbc, &hstmt); \
-  } \
-\
-  (void)free_basic_handles(&henv, &hdbc, &hstmt); \
-\
+      if ((rc == FAIL) && (FAIL != tests[i].expect)) \
+        ++failcnt; \
+      SKIP_REASON= NULL; /* Reset SKIP_REASON */ \
+      (void)free_basic_handles(&henv, &hdbc, &hstmt); \
+      mem_gc_flush(); \
+    } \
+  }
+
+#define RUN_TESTS \
+  RUN_TESTS_ONCE \
+  mem_gc_flush(); \
   exit(failcnt); \
 }
 
@@ -259,6 +384,11 @@ int main(int argc, char **argv) \
     return SKIP; \
   } while (0)
 
+#ifdef CPP_THROW_CLASS
+#define TEST_RETURN_FAIL throw CPP_THROW_CLASS(__FILE__, __LINE__);
+#else
+#define TEST_RETURN_FAIL return FAIL;
+#endif
 
 /**
   Execute an SQL statement and bail out if the execution does not return
@@ -274,7 +404,7 @@ do { \
              "SQLExecDirect(" #statement ", \"" query "\", SQL_NTS)",\
              __FILE__, __LINE__); \
   if (rc != SQL_SUCCESS && rc != SQL_SUCCESS_WITH_INFO) \
-    return FAIL; \
+    TEST_RETURN_FAIL; \
 } while (0)
 
 
@@ -296,7 +426,7 @@ do { \
                __FILE__, __LINE__); \
     printf("# Expected %d, but got %d in %s on line %d\n", expect, rc, \
            __FILE__, __LINE__); \
-    return FAIL; \
+    TEST_RETURN_FAIL; \
   } \
 } while (0)
 
@@ -310,10 +440,10 @@ do { \
 */
 #define ok_stmt(statement, call) \
 do { \
-  SQLRETURN rc= (call); \
-  print_diag(rc, SQL_HANDLE_STMT, (statement), #call, __FILE__, __LINE__); \
-  if (rc != SQL_SUCCESS && rc != SQL_SUCCESS_WITH_INFO) \
-    return FAIL; \
+  SQLRETURN _rc= (call); \
+  print_diag(_rc, SQL_HANDLE_STMT, (statement), #call, __FILE__, __LINE__); \
+  if (_rc != SQL_SUCCESS && _rc != SQL_SUCCESS_WITH_INFO) \
+    TEST_RETURN_FAIL; \
 } while (0)
 
 
@@ -329,7 +459,7 @@ do { \
   SQLRETURN rc= (call); \
   print_diag(rc, SQL_HANDLE_DESC, (desc), #call, __FILE__, __LINE__); \
   if (rc != SQL_SUCCESS && rc != SQL_SUCCESS_WITH_INFO) \
-    return FAIL; \
+    TEST_RETURN_FAIL; \
 } while (0)
 
 
@@ -350,7 +480,7 @@ do { \
     print_diag(rc, (type), (hnd), #call, __FILE__, __LINE__); \
     printf("# Expected %d, but got %d in %s on line %d\n", (expect), rc, \
            __FILE__, __LINE__); \
-    return FAIL; \
+    TEST_RETURN_FAIL; \
   } \
 } while (0)
 
@@ -380,10 +510,10 @@ do { \
 */
 #define ok_env(environ, call) \
 do { \
-  SQLRETURN rc= (call); \
-  print_diag(rc, SQL_HANDLE_ENV, (environ), #call, __FILE__, __LINE__); \
-  if (rc != SQL_SUCCESS && rc != SQL_SUCCESS_WITH_INFO) \
-    return FAIL; \
+  SQLRETURN _rc= (call); \
+  print_diag(_rc, SQL_HANDLE_ENV, (environ), #call, __FILE__, __LINE__); \
+  if (_rc != SQL_SUCCESS && _rc != SQL_SUCCESS_WITH_INFO) \
+    TEST_RETURN_FAIL; \
 } while (0)
 
 
@@ -396,10 +526,25 @@ do { \
 */
 #define ok_con(con, call) \
 do { \
-  SQLRETURN rc= (call); \
-  print_diag(rc, SQL_HANDLE_DBC, (con), #call, __FILE__, __LINE__); \
-  if (rc != SQL_SUCCESS && rc != SQL_SUCCESS_WITH_INFO) \
-    return FAIL; \
+  SQLRETURN _rc= (call); \
+  print_diag(_rc, SQL_HANDLE_DBC, (con), #call, __FILE__, __LINE__); \
+  if (_rc != SQL_SUCCESS && _rc != SQL_SUCCESS_WITH_INFO) \
+    TEST_RETURN_FAIL; \
+} while (0)
+
+
+/**
+  Verify that the results of an ODBC function call on an environment handle
+  was SQL_SUCCESS or SQL_SUCCESS_WITH_INFO.
+
+  @param call    The function call
+*/
+#define ok_install(call) \
+do { \
+  BOOL _rc= (call); \
+  print_diag_installer(_rc, #call, __FILE__, __LINE__); \
+  if (!_rc) \
+    TEST_RETURN_FAIL; \
 } while (0)
 
 
@@ -415,7 +560,7 @@ do { \
   if (!(a)) { \
     printf("# !(%s) in %s on line %d\n", \
            #a, __FILE__, __LINE__); \
-    return FAIL; \
+    TEST_RETURN_FAIL; \
   } \
 } while (0);
 
@@ -434,11 +579,11 @@ do { \
 #define is_str(a, b, c) \
 do { \
   char *val_a= (char *)(a), *val_b= (char *)(b); \
-  int val_len= (int)(c); \
+  int val_len= (int)(c) == SQL_NTS ? strlen(val_a) : (int)(c); \
   if (strncmp(val_a, val_b, val_len) != 0) { \
     printf("# %s ('%*s') != '%*s' in %s on line %d\n", \
            #a, val_len, val_a, val_len, val_b, __FILE__, __LINE__); \
-    return FAIL; \
+    TEST_RETURN_FAIL; \
   } \
 } while (0);
 
@@ -452,12 +597,12 @@ do { \
 */
 #define is_wstr(a, b, c) \
 do { \
-  wchar_t *val_a= (a), *val_b= (b); \
+  SQLWCHAR *val_a= (a), *val_b= (b); \
   int val_len= (int)(c); \
-  if (memcmp(val_a, val_b, val_len * sizeof(wchar_t)) != 0) { \
+  if (memcmp(val_a, val_b, val_len * sizeof(SQLWCHAR)) != 0) { \
     printf("# %s ('%*ls') != '%*ls' in %s on line %d\n", \
            #a, val_len, val_a, val_len, val_b, __FILE__, __LINE__); \
-    return FAIL; \
+    TEST_RETURN_FAIL; \
   } \
 } while (0);
 
@@ -470,12 +615,13 @@ do { \
 */
 #define is_num(a, b) \
 do { \
-  if (a != b) { \
+  long long a1= (a), a2= (b); \
+  if (a1 != a2) { \
     printf("# %s (%lld) != %lld in %s on line %d\n", \
-           #a, (long long)a, (long long)b, __FILE__, __LINE__); \
-    return FAIL; \
+           #a, a1, a2, __FILE__, __LINE__); \
+    TEST_RETURN_FAIL; \
   } \
-} while (0);
+} while (0)
 
 
 /* convenience macro for using statement */
@@ -500,10 +646,29 @@ int check_sqlstate_ex(SQLHANDLE hnd, SQLSMALLINT hndtype, char *sqlstate)
 }
 
 
+int check_errmsg_ex(SQLHANDLE hnd, SQLSMALLINT hndtype, const char *msg)
+{
+  SQLCHAR     sql_state[6];
+  SQLINTEGER  err_code= 0;
+  SQLCHAR     err_msg[SQL_MAX_MESSAGE_LENGTH]= {0};
+  SQLSMALLINT err_len= 0;
+
+  memset(err_msg, 'C', SQL_MAX_MESSAGE_LENGTH);
+  SQLGetDiagRec(hndtype, hnd, 1, sql_state, &err_code, err_msg,
+                SQL_MAX_MESSAGE_LENGTH - 1, &err_len);
+
+  is(strstr((char*)err_msg, msg) != NULL);
+
+  return OK;
+}
+
+
 /**
+  Print error and diagnostic information for ODBC API functions that did not
+  finish with SQL_SUCCESS(_WITH_INFO) result
 */
 static void print_diag(SQLRETURN rc, SQLSMALLINT htype, SQLHANDLE handle,
-		       const char *text, const char *file, int line)
+           const char *text, const char *file, int line)
 {
   if (rc != SQL_SUCCESS)
   {
@@ -519,7 +684,7 @@ static void print_diag(SQLRETURN rc, SQLSMALLINT htype, SQLHANDLE handle,
     drc= SQLGetDiagRec(htype, handle, 1, sqlstate, &native_error,
                        message, SQL_MAX_MESSAGE_LENGTH - 1, &length);
 
-    if (SQL_SUCCEEDED(drc))
+    if (SQL_IS_SUCCESS(drc))
       printf("# [%6s] %*s in %s on line %d\n",
              sqlstate, length, message, file, line);
     else
@@ -529,6 +694,30 @@ static void print_diag(SQLRETURN rc, SQLSMALLINT htype, SQLHANDLE handle,
 }
 
 
+/**
+  Print error and diagnostic information for ODBC INSTALLER API functions
+  that did not return TRUE (1)
+*/
+static void print_diag_installer(BOOL is_success, const char *text,
+                                 const char *file, int line)
+{
+  if (!is_success)
+  {
+    DWORD  error_code;
+    WORD length;
+    char     message[SQL_MAX_MESSAGE_LENGTH];
+    SQLRETURN   drc;
+    drc= SQLInstallerError(1, &error_code, message, SQL_MAX_MESSAGE_LENGTH - 1, &length);
+    if (SQL_IS_SUCCESS(drc))
+      printf("# [%s] %s in %s on line %d\n",
+             text, message, file, line);
+    else
+      printf("# Did not get expected diagnostics from SQLInstallerError() = %d"
+             " in file %s on line %d\n", drc, file, line);
+  }
+}
+
+
 /* UTILITY MACROS */
 #define myenv(henv,r)  \
   do { \
@@ -606,7 +795,7 @@ static void print_diag(SQLRETURN rc, SQLSMALLINT htype, SQLHANDLE handle,
 /**
   Print resultset dashes
 */
-static int my_print_dashes(SQLHSTMT hstmt, SQLSMALLINT nCol)
+int my_print_dashes(SQLHSTMT hstmt, SQLSMALLINT nCol)
 {
     SQLLEN     disp_size, nullable;
     SQLCHAR    ColName[MAX_NAME_LEN+1];
@@ -684,12 +873,12 @@ static int my_print_data(SQLHSTMT hstmt, SQLUSMALLINT index,
 /* If we just return FAIL from my_print_non_format_result - it will
    considered as 1 row by caller */
 #define mystmt_rows(hstmt,r, row)  \
-	do { \
-	print_diag(r, SQL_HANDLE_STMT, (hstmt), "mystmt_row(hstmt,r,row)", \
-	__FILE__, __LINE__); \
-	if (!SQL_SUCCEEDED(r)) \
-	  return row; \
-	} while (0)
+  do { \
+  print_diag(r, SQL_HANDLE_STMT, (hstmt), "mystmt_row(hstmt,r,row)", \
+  __FILE__, __LINE__); \
+  if (!SQL_IS_SUCCESS(r)) \
+    return row; \
+  } while (0)
 /**
 RESULT SET
 */
@@ -701,9 +890,10 @@ int my_print_non_format_result(SQLHSTMT hstmt)
     SQLCHAR     szColName[MAX_NAME_LEN+1];
     SQLCHAR     szData[MAX_COLUMNS][MAX_ROW_DATA_LEN]={{0}};
     SQLSMALLINT nIndex,ncol= 0,pfSqlType, pcbScale, pfNullable;
+    SQLLEN      ind_strlen;
 
     rc = SQLNumResultCols(hstmt,&ncol);
-    
+
     mystmt_rows(hstmt,rc,-1);
 
     for (nIndex = 1; nIndex <= ncol; ++nIndex)
@@ -717,14 +907,14 @@ int my_print_non_format_result(SQLHSTMT hstmt)
         fprintf(stdout, "%s\t", szColName);
 
         rc = SQLBindCol(hstmt,nIndex, SQL_C_CHAR, szData[nIndex-1],
-                        MAX_ROW_DATA_LEN+1,NULL);
+                        MAX_ROW_DATA_LEN+1,&ind_strlen);
         mystmt_rows(hstmt,rc,-nIndex);
     }
 
     fprintf(stdout,"\n");
 
     rc = SQLFetch(hstmt);
-    while (SQL_SUCCEEDED(rc))
+    while (SQL_IS_SUCCESS(rc))
     {
         ++nRowCount;
         for (nIndex=0; nIndex< ncol; ++nIndex)
@@ -815,7 +1005,7 @@ SQLUINTEGER myrowcount(SQLHSTMT hstmt)
     rc = SQLFetch(hstmt);
     while (rc == SQL_SUCCESS || rc == SQL_SUCCESS_WITH_INFO)
     {
-        nRowCount++;
+        ++nRowCount;
         rc = SQLFetch(hstmt);
     }
     SQLFreeStmt(hstmt,SQL_UNBIND);
@@ -847,8 +1037,16 @@ SQLINTEGER my_fetch_int(SQLHSTMT hstmt, SQLUSMALLINT icol)
     SQLLEN len;
 
     SQLGetData(hstmt, icol, SQL_INTEGER, &nData, 0, &len);
-    printMessage("my_fetch_int: %ld (%ld)", (long int)nData, len);
-    return (len != SQL_NULL_DATA) ? nData : 0;
+    if (len == SQL_NULL_DATA)
+    {
+      printMessage("my_fetch_int: NULL");
+    }
+    else
+    {
+      printMessage("my_fetch_int: %ld (%ld)", (long int)nData, len);
+      return nData;
+    }
+    return 0;
 }
 
 
@@ -874,7 +1072,13 @@ const char *my_fetch_str(SQLHSTMT hstmt, SQLCHAR *szData,SQLUSMALLINT icol)
     SQLLEN nLen;
 
     SQLGetData(hstmt,icol,SQL_CHAR,szData,MAX_ROW_DATA_LEN+1,&nLen);
-    printMessage(" my_fetch_str: %s(%ld)",szData,nLen);
+    /* If Null value - putting down smth meaningful. also that allows caller to
+       better/(in more easy way) test the value */
+    if (nLen < 0)
+    {
+      strcpy((char*)szData, "(Null)");
+    }
+    printMessage(" my_fetch_str: %s(%ld)", szData, nLen);
     return((const char *)szData);
 }
 
@@ -908,7 +1112,7 @@ wchar_t *my_fetch_wstr(SQLHSTMT hstmt, SQLWCHAR *buffer, SQLUSMALLINT icol)
   SQLLEN nLen;
 
   rc= SQLGetData(hstmt, icol, SQL_WCHAR, buffer, MAX_ROW_DATA_LEN + 1, &nLen);
-  if (!SQL_SUCCEEDED(rc))
+  if (!SQL_IS_SUCCESS(rc))
     return L"";
 
   buffer[nLen/sizeof(SQLWCHAR)]= 0;
@@ -922,15 +1126,15 @@ wchar_t *my_fetch_wstr(SQLHSTMT hstmt, SQLWCHAR *buffer, SQLUSMALLINT icol)
 */
 int driver_supports_setpos(SQLHDBC hdbc)
 {
-    SQLRETURN    rc;
-    SQLUSMALLINT status= TRUE;
+  SQLRETURN    rc;
+  SQLUSMALLINT status= TRUE;
 
-    rc = SQLGetFunctions(hdbc, SQL_API_SQLSETPOS, &status);
-    mycon(hdbc,rc);
+  rc = SQLGetFunctions(hdbc, SQL_API_SQLSETPOS, &status);
+  mycon(hdbc,rc);
 
-    if (!status)
-        return FALSE;
-    return TRUE;
+  if (!status)
+    return FALSE;
+  return TRUE;
 }
 
 /*
@@ -938,16 +1142,27 @@ int driver_supports_setpos(SQLHDBC hdbc)
 */
 int mysql_min_version(SQLHDBC hdbc, char *min_version, unsigned int length)
 {
-    SQLCHAR server_version[MYSQL_NAME_LEN+1];
-    SQLRETURN rc;
+  char server_version[MYSQL_NAME_LEN+1];
+  SQLRETURN rc;
 
-    rc = SQLGetInfo(hdbc,SQL_DBMS_VER,server_version,MYSQL_NAME_LEN,NULL);
-    mycon(hdbc, rc);
+  rc = SQLGetInfo(hdbc,SQL_DBMS_VER,server_version,MYSQL_NAME_LEN,NULL);
+  mycon(hdbc, rc);
 
-    if (strncmp((char *)server_version, min_version, length) >= 0)
-        return TRUE;
+  {
+    unsigned int major1= 0, major2= 0, minor1= 0, minor2= 0, build1= 0, build2= 0;
 
-    return FALSE;
+    sscanf(server_version, "%u.%u.%u", &major1, &minor1, &build1);
+    sscanf(min_version, "%u.%u.%u", &major2, &minor2, &build2);
+
+    if ( major1 > major2 ||
+      major1 == major2 && (minor1 > minor2 ||
+                          minor1 ==  minor2 && build1 >= build2))
+    {
+      return TRUE;
+    }
+  }
+
+  return FALSE;;
 }
 
 /*
@@ -955,15 +1170,15 @@ int mysql_min_version(SQLHDBC hdbc, char *min_version, unsigned int length)
 */
 int server_supports_trans(SQLHDBC hdbc)
 {
-    SQLSMALLINT trans;
-    SQLRETURN   rc;
+  SQLSMALLINT trans;
+  SQLRETURN   rc;
 
-    rc = SQLGetInfo(hdbc,SQL_TXN_CAPABLE,&trans,0,NULL);
-    mycon(hdbc,rc);
+  rc = SQLGetInfo(hdbc,SQL_TXN_CAPABLE,&trans,0,NULL);
+  mycon(hdbc,rc);
 
-    if (trans != SQL_TC_NONE)
-        return TRUE;
-    return FALSE;
+  if (trans != SQL_TC_NONE)
+      return TRUE;
+  return FALSE;
 }
 
 
@@ -994,37 +1209,184 @@ int mydrvconnect(SQLHENV *henv, SQLHDBC *hdbc, SQLHSTMT *hstmt, SQLCHAR *connIn)
 }
 
 
-int alloc_basic_handles(SQLHENV *henv, SQLHDBC *hdbc, SQLHSTMT *hstmt)
+/*
+  Helper function to make the connection string.
+*/
+SQLCHAR *make_conn_str(const SQLCHAR *dsn, const SQLCHAR *uid,
+                       const SQLCHAR *pwd, const SQLCHAR *db,
+                       const SQLCHAR *options, int hide_password)
 {
-  SQLCHAR   connIn[MAX_NAME_LEN+1], connOut[MAX_NAME_LEN+1];
+  static SQLCHAR connIn[4096]= {0};
+  SQLCHAR dsn_buf[MAX_NAME_LEN]= {0};
+  SQLCHAR socket_buf[MAX_NAME_LEN]= {0};
+  SQLCHAR server_buf[MAX_NAME_LEN] = {0};
+  /* ";database="+ we make buffer bigger for one certain test */
+  SQLCHAR     db_buf[4096]= {0};
+
+  /* Should fit 8 byte + ";port=" */
+  SQLCHAR     port_buf[32]= {0};
+  BOOL skip_socket = 0;
+
+  /* We never set the custom DSN, but sometimes use DRIVER instead */
+  if (dsn == NULL)
+    snprintf((char *)dsn_buf, sizeof(dsn_buf), "DSN=%s", (char *)mydsn);
+  else if (dsn == (const SQLCHAR*)USE_DRIVER)
+    snprintf((char *)dsn_buf, sizeof(dsn_buf), "DRIVER=%s", (char *)mydriver);
+  else
+    snprintf((char *)dsn_buf, sizeof(dsn_buf), "DSN=%s", (char *)dsn);
+
+  if (uid     == NULL) uid=     myuid;
+  if (pwd     == NULL) pwd=     mypwd;
+  if (db      == NULL) db=      mydb;
+  if (options == NULL) options= my_str_options;
+
+  if (hide_password)
+    pwd = (SQLCHAR*)"*************";
+
+  snprintf((char *)connIn, sizeof(connIn), "%s;UID=%s;PWD=%s;OPTION=%d;",
+          (char *)dsn_buf, (char *)uid, (char *)pwd, myoption);
+
+  if (strstr((const char*)options, "SOCKET=;") &&
+      strcmp((const char*)myserver, "localhost") == 0)
+  {
+    skip_socket = 1;
+    char *port = NULL;
+    if ((port = getenv("TEST_PORT")) ||
+        (port = getenv("MYSQL_PORT")))
+    {
+      myport = atoi(port);
+    }
+    strncat((char *)connIn, (char*)";SERVER=127.0.0.1;", sizeof(connIn) - 1);
+  }
+  else
+  {
+    snprintf((char *)server_buf, sizeof(server_buf), ";SERVER=%s", myserver);
+    strncat((char *)connIn, (char *)server_buf, sizeof(connIn) - 1);
+  }
+
+  if (mysock && mysock[0] && skip_socket == 0)
+  {
+    snprintf((char *)socket_buf, sizeof(socket_buf), ";SOCKET=%s", (char *)mysock);
+    strncat((char *)connIn, (char*)socket_buf, sizeof(connIn) - 1);
+  }
+  if (db && db[0])
+  {
+    snprintf((char *)db_buf, sizeof(db_buf), ";DATABASE=%s", (char *)db);
+    strncat((char *)connIn, (char *)db_buf, sizeof(connIn) - 1);
+  }
+  if (myport)
+  {
+    snprintf((char*)port_buf, sizeof(port_buf), ";PORT=%d", myport);
+    strncat((char *)connIn, (char*)port_buf, sizeof(connIn) - 1);
+  }
+
+  if (options != NULL && options[0] > 0)
+  {
+    strncat((char*)connIn, ";", sizeof(connIn));
+    strncat((char*)connIn, (char*)options, sizeof(connIn) - 1);
+  }
+
+#if MYSQL_VERSION_ID >= 50507
+  if (init_auth_plugin)
+  {
+    init_auth_plugin= 0; /* reset the plugin init flag */
+    if (myauth && myauth[0])
+    {
+      strncat((char *)connIn, ";DEFAULTAUTH=", sizeof(connIn));
+      strncat((char *)connIn, (char *)myauth, sizeof(connIn) - 1);
+    }
+    if (myplugindir && myplugindir[0])
+    {
+      strncat((char *)connIn, ";PLUGINDIR=", sizeof(connIn));
+      strncat((char *)connIn, (char *)myplugindir, sizeof(connIn) - 1);
+    }
+  }
+#endif
+  return connIn;
+}
+
+
+/* Helper function for tests to get (additional) connection
+   If dsn, uid, pwd or options is null - they defualt to mydsn, myuid, mypwd
+   and my_str_options, respectively.
+   myoption, mysock and myport values are used. */
+int get_connection(SQLHDBC *hdbc, const SQLCHAR *dsn, const SQLCHAR *uid,
+                   const SQLCHAR *pwd, const SQLCHAR *db,
+                   const SQLCHAR *options)
+{
+  /* Buffers have to be large enough to contain SSL options and long names */
+  SQLCHAR     connOut[4096];
   SQLSMALLINT len;
+  SQLRETURN   rc;
+  SQLCHAR     driver_name[16]; /* Should be enough for myodbc library file name */
+  SQLCHAR     *connIn;
+
+  connIn = make_conn_str(dsn, uid, pwd, db, options, 0);
+  rc= SQLDriverConnect(*hdbc, NULL, connIn, SQL_NTS, connOut,
+                       MAX_NAME_LEN, &len, SQL_DRIVER_NOPROMPT);
+
+  if (!SQL_IS_SUCCESS(rc))
+  {
+    /* re-build and print the connection string with hidden password */
+    printf("# Connection failed with the following Connection string: " \
+           "\n%s\n", (char*)make_conn_str(dsn, uid, pwd, db, options, 1));
+    return rc;
+  }
+
+  /* Let's neglect possibility that error returned by get_connection() can be
+     in fact error of turning autocommit on */
+  rc= SQLSetConnectAttr(*hdbc, SQL_ATTR_AUTOCOMMIT,
+                                  (SQLPOINTER)SQL_AUTOCOMMIT_ON, 0);
+  if (!SQL_IS_SUCCESS(rc))
+  {
+    return rc;
+  }
+
+  if (unicode_driver < 0)
+  {
+    rc= SQLGetInfo(*hdbc, SQL_DRIVER_NAME, driver_name, sizeof(driver_name), NULL);
+
+    if (SQL_IS_SUCCESS(rc))
+    {
+      /* ANSI driver file name contains 5a.{dll|so} */
+      unicode_driver= strstr((char*)driver_name, "a.") == NULL ? 1 : 0;
+    }
+  }
+
+  return rc;
+}
+
+
+int alloc_basic_handles_with_opt(SQLHENV *henv, SQLHDBC *hdbc,
+                                 SQLHSTMT *hstmt,  const SQLCHAR *dsn,
+                                 const SQLCHAR *uid, const SQLCHAR *pwd,
+                                 const SQLCHAR *db, const SQLCHAR *options)
+{
+  if (myenable_pooling)
+  {
+    SQLSetEnvAttr(NULL, SQL_ATTR_CONNECTION_POOLING,
+                 (SQLPOINTER)SQL_CP_ONE_PER_DRIVER, 0);
+  }
 
   ok_env(*henv, SQLAllocHandle(SQL_HANDLE_ENV, SQL_NULL_HANDLE, henv));
 
   ok_env(*henv, SQLSetEnvAttr(*henv, SQL_ATTR_ODBC_VERSION,
                               (SQLPOINTER)SQL_OV_ODBC3, 0));
+  if (myenable_pooling)
+  {
+    SQLRETURN rc= SQLSetEnvAttr(*henv, SQL_ATTR_CP_MATCH, (SQLPOINTER)SQL_CP_STRICT_MATCH, 0);
+
+    if( !SQL_IS_SUCCESS(rc))
+    {
+      /* No DM case? */
+      is_num(check_sqlstate_ex(*henv, SQL_HANDLE_ENV, "HYC00"), OK);
+    }
+  }
 
   ok_env(*henv, SQLAllocHandle(SQL_HANDLE_DBC, *henv, hdbc));
 
-  sprintf((char *)connIn, "DSN=%s;UID=%s;PWD=%s;DATABASE=test;OPTION=%d",
-          (char *)mydsn, (char *)myuid, (char *)mypwd, myoption);
-  if (mysock && mysock[0])
-  {
-    strcat((char *)connIn, ";SOCKET=");
-    strcat((char *)connIn, (char *)mysock);
-  }
-  if (myport)
-  {
-    char buff[20];
-    sprintf(buff, ";PORT=%d", myport);
-    strcat((char *)connIn, buff);
-  }
-
-  ok_con(*hdbc, SQLDriverConnect(*hdbc, NULL, connIn, SQL_NTS, connOut,
-                                 MAX_NAME_LEN, &len, SQL_DRIVER_NOPROMPT));
-
-  ok_con(*hdbc, SQLSetConnectAttr(*hdbc, SQL_ATTR_AUTOCOMMIT,
-                                  (SQLPOINTER)SQL_AUTOCOMMIT_ON, 0));
+  /* ok_con(*hdbc, SQLConnect(*hdbc, mydsn, strlen(mydsn), myuid, strlen(myuid), mypwd, strlen(mypwd))); */
+  ok_con(*hdbc, get_connection(hdbc, dsn, uid, pwd, db, options));
 
   ok_con(*hdbc, SQLAllocHandle(SQL_HANDLE_STMT, *hdbc, hstmt));
 
@@ -1032,18 +1394,80 @@ int alloc_basic_handles(SQLHENV *henv, SQLHDBC *hdbc, SQLHSTMT *hstmt)
 }
 
 
+int alloc_basic_handles(SQLHENV *henv, SQLHDBC *hdbc, SQLHSTMT *hstmt)
+{
+  return alloc_basic_handles_with_opt(henv, hdbc, hstmt, (SQLCHAR *)mydsn,
+                                      (SQLCHAR *)myuid, (SQLCHAR *)mypwd,
+                                      (SQLCHAR *)mydb,
+                                      (SQLCHAR *)my_str_options);
+}
+
+
 int free_basic_handles(SQLHENV *henv,SQLHDBC *hdbc, SQLHSTMT *hstmt)
 {
-  /* We don't care if this succeeds, the connection may have gone away. */
-  (void)SQLEndTran(SQL_HANDLE_DBC, *hdbc, SQL_COMMIT);
 
-  ok_stmt(*hstmt, SQLFreeStmt(*hstmt, SQL_DROP));
+  if(hstmt && *hstmt)
+  {
+    ok_stmt(*hstmt, SQLFreeStmt(*hstmt, SQL_DROP));
+    *hstmt = NULL;
+  }
 
-  ok_con(*hdbc, SQLDisconnect(*hdbc));
+  if(hdbc && *hdbc)
+  {
+    /* We don't care if this succeeds, the connection may have gone away. */
+    (void)SQLEndTran(SQL_HANDLE_DBC, *hdbc, SQL_COMMIT);
+    ok_con(*hdbc, SQLDisconnect(*hdbc));
+    ok_con(*hdbc, SQLFreeConnect(*hdbc));
+    *hdbc = NULL;
+  }
 
-  ok_con(*hdbc, SQLFreeConnect(*hdbc));
+  if(henv && *henv)
+  {
+    ok_env(*henv, SQLFreeEnv(*henv));
+    *henv = NULL;
+  }
 
-  ok_env(*henv, SQLFreeEnv(*henv));
+  return OK;
+}
+
+
+/*
+  Allocate size bytes and return a pointer
+  to allocated memory with storing in
+  garbage collector
+*/
+void *gc_alloc(size_t len)
+{
+  if (gc_blk.counter >= MAX_MEM_BLOCK_ELEMENTS || len <= 0)
+  {
+    printf("# GC Memory reached maximum limit counter: %d " \
+        "or length:%d less then equal to 0 in %s on line %d\n",
+        gc_blk.counter, (int)len, __FILE__, __LINE__); \
+    return NULL;
+  }
+
+  gc_blk.blk[gc_blk.counter]= malloc(len);
+  return gc_blk.blk[gc_blk.counter++];
+}
+
+
+/*
+  Free allocated memory collected by
+  garbage collector
+*/
+int mem_gc_flush()
+{
+  int i= 0;
+  if (gc_blk.counter <= 0)
+  {
+    return FAIL;
+  }
+
+  while (i < gc_blk.counter)
+  {
+    free(gc_blk.blk[i]);
+    gc_blk.blk[i++]= 0;
+  }
 
   return OK;
 }
@@ -1068,14 +1492,14 @@ SQLWCHAR *dup_wchar_t_as_sqlwchar(wchar_t *from, size_t len)
 {
   if (sizeof(wchar_t) == sizeof(SQLWCHAR))
   {
-    SQLWCHAR *to= malloc(len * sizeof(SQLWCHAR));
+    SQLWCHAR *to= (SQLWCHAR*)gc_alloc(len * sizeof(SQLWCHAR));
     memcpy(to, from, len * sizeof(wchar_t));
     return to;
   }
   else
   {
     size_t i;
-    SQLWCHAR *to= malloc(2 * len * sizeof(SQLWCHAR));
+    SQLWCHAR *to= (SQLWCHAR*)gc_alloc(2 * len * sizeof(SQLWCHAR));
     SQLWCHAR *out= to;
     for (i= 0; i < len; i++)
       to+= utf32toutf16((UTF32)from[i], (UTF16 *)to);
@@ -1098,7 +1522,7 @@ SQLWCHAR *dup_wchar_t_as_sqlwchar(wchar_t *from, size_t len)
 */
 SQLWCHAR *dup_char_as_sqlwchar(SQLCHAR *from)
 {
-  SQLWCHAR *to= malloc((strlen((char *)from) + 1) * sizeof(SQLWCHAR));
+  SQLWCHAR *to= (SQLWCHAR*)gc_alloc((strlen((char *)from) + 1) * sizeof(SQLWCHAR));
   SQLWCHAR *out= to;
   while (from && *from)
     *(to++)= (SQLWCHAR)*(from++);
@@ -1134,8 +1558,9 @@ int using_unixodbc_version(SQLHANDLE henv, SQLCHAR *ver)
   SQLCHAR buf[10];
   if(SQLGetEnvAttr(henv, SQL_ATTR_UNIXODBC_VERSION, buf, 10, NULL) != SQL_SUCCESS)
     return 0;
-  if(!strcmp(buf, ver))
+  if(!strcmp((char*)buf, (char*)ver))
     return 1;
 #endif /* SQL_ATTR_UNIXODBC_VERSION */
   return 0;
 }
+
